const { app, BrowserWindow, shell, session, ipcMain } = require("electron");
const path = require("path");
const Store = require("electron-store");
const Badge = require("./Badge");
const store = new Store({ defaults: { flash_enabled: true } });

let mainWindow;

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (require("electron-squirrel-startup")) {
	app.quit();
}

const createWindow = () => {
	// Create the browser window.
	const width = store.get("winBounds")?.width || 800;
	const height = store.get("winBounds")?.height || 600;
	// const { totalWidth, totalHeight } = screen.getAllDisplays()?.reduce?.(
	// 	(acc, screen) => {
	// 		acc.totalWidth += screen.workAreaSize.width;
	// 		acc.totalHeight += screen.workAreaSize.height;

	// 		return acc;
	// 	},
	// 	{ totalWidth: 0, totalHeight: 0 }
	// );

	mainWindow = new BrowserWindow({
		width: width,
		height: height,
		minHeight: 400,
		minWidth: 400,
		// x: Math.max(0, Math.min(store.get("winBounds").x, totalWidth - width )),
		// y:Math.max(0, Math.min(store.get("winBounds").y, totalHeight  / height)),
		x: store.get("winBounds")?.x,
		y: store.get("winBounds")?.y,

		titleBarStyle: "hidden",
		icon: "assets/sellfinity-icon.png",
		webPreferences: {
			preload: path.join(__dirname, "preload.js"),
			nodeIntegration: true,
			enableRemoteModule: true,
		},
		titleBarOverlay: {
			color: "#15182e",
			symbolColor: "#F0F0F0",
			height: 50,
		},
		frame: false,
		trafficLightPosition: { x: 15, y: 15 },
		// hasShadow: false,
		// webPreferences: {
		// 	nodeIntegration: true,
		// },
	});

	if (process.platform === "win32" || process.platform === "win64") {
		const badgeOptions = {};
		new Badge(mainWindow, badgeOptions);
	}

	// and load the index.html of the app.
	// mainWindow.loadFile(path.join(__dirname, 'index.html'));
	// mainWindow.loadURL("http://localhost:3001");
	mainWindow.loadURL("https://chat.sellfinity.com/");
	//
	// mainWindow.once("ready-to-show", () => {
	mainWindow.show();
	// });

	mainWindow.on("close", () => {
		store.set("winBounds", mainWindow.getBounds());
	});

	mainWindow.webContents.setWindowOpenHandler(({ url }) => {
		shell.openExternal(url);
		return { action: "deny" };
	});

	// Open the DevTools.
	// mainWindow.webContents.openDevTools();
};

if (!app.getLoginItemSettings()?.launchItems?.length) {
	app.setLoginItemSettings({ ...(app.getLoginItemSettings() || {}), openAtLogin: true, executableWillLaunchAtLogin: true, enabled: true });
}

ipcMain.on("setLoginItemSettings", (event, settings) => {
	app.setLoginItemSettings({ ...(app.getLoginItemSettings() || {}), ...(settings || {}) });
});

ipcMain.on("toggleMaximize", (event, settings) => {});

ipcMain.handle("getLoginItemSettings", async (event, options) => {
	return app.getLoginItemSettings();
});

ipcMain.handle("getStore", (event, key) => {
	return store.store;
});

ipcMain.handle("getName", async (event, key) => {
	return app.getName();
});

ipcMain.handle("getVersion", async (event, key) => {
	return app.getVersion();
});

ipcMain.handle("getStoreValue", (event, key) => {
	return store.get(key);
});

ipcMain.on("setStoreValue", (event, opt) => {
	store.set(opt);
});

ipcMain.on("setBadgeCount", (event, count) => {
	app.dock?.setBadge?.(count ? String(count) : "");
	app?.setBadge?.(count ? String(count) : "");
});

ipcMain.on("flash", () => {
	if (store.get("flash_enabled")) {
		if (process.platform === "darwin") {
			app.dock.bounce();
		}
		mainWindow.flashFrame?.(true);
	}
});

ipcMain.on("close", () => {
	mainWindow.close();
});

ipcMain.on("destroy", () => {
	store.set("winBounds", mainWindow.getBounds());
	mainWindow.destroy();
});

const gotTheLock = app.requestSingleInstanceLock();

if (!gotTheLock) {
	app.quit();
} else {
	app.on("second-instance", (event, commandLine, workingDirectory) => {
		// Someone tried to run a second instance, we should focus our window.
		if (mainWindow) {
			if (mainWindow.isMinimized()) mainWindow.restore();
			mainWindow.focus();
		}
	});

	// Create mainWindow, load the rest of the app, etc...
	if (app.isReady()) {
		createWindow();
	} else {
		app.whenReady().then(createWindow);
	}
}

// // This method will be called when Electron has finished
// // initialization and is ready to create browser windows.
// // Some APIs can only be used after this event occurs.
// app.on("ready", createWindow);

// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on("window-all-closed", () => {
	if (process.platform !== "darwin") {
		app.quit();
	}
});

app.on("activate", () => {
	// On OS X it's common to re-create a window in the app when the
	// dock icon is clicked and there are no other windows open.
	if (BrowserWindow.getAllWindows().length === 0) {
		createWindow();
	}
});

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and import them here.
